@@ -10,6 +10,7 @@ from statefun_tasks.utils import _task_type_for, _unpack_single_tuple_args, _gen
 from statefun_tasks.pipeline import _Pipeline
 from statefun_tasks.tasks import FlinkTask
 from statefun_tasks.task_impl.handlers import TaskRequestHandler, TaskResponseHandler, TaskActionHandler, ChildPipelineHandler
+from statefun_tasks.events import EventHandlers
 
 from statefun.request_reply import BatchContext
 from typing import Union
@@ -40,6 +41,7 @@ class FlinkTasks(object):
         self._egress_type_name = egress_type_name
         self._serialiser = serialiser if serialiser is not None else DefaultSerialiser()
         self._bindings = {}
+        self._events = EventHandlers()
 
         self.register_builtin('run_pipeline', _run_pipeline)
 
@@ -50,6 +52,13 @@ class FlinkTasks(object):
             ChildPipelineHandler()
         ]
 
+    @property
+    def events(self) -> EventHandlers:
+        """
+        EventHandler for this FlinkTasks instance
+        """
+        return self._events
+
     def value_specs(self):
         return self._value_specs
 
@@ -62,7 +71,7 @@ class FlinkTasks(object):
         :param fun: a function, partial or lambda representing the built in
         :param params: any additional parameters to the built in
         """
-        self._bindings[f'__builtins.{type_name}'] = FlinkTask(fun, self._serialiser, **params)
+        self._bindings[f'__builtins.{type_name}'] = FlinkTask(fun, self._serialiser, self._events, **params)
 
     def register(self, fun, wrapper=None, module_name=None, **params):
         """
@@ -79,7 +88,7 @@ class FlinkTasks(object):
             raise ValueError("function instance must be provided")
 
         fun.type_name = _task_type_for(fun, module_name)
-        self._bindings[fun.type_name] = FlinkTask(wrapper or fun, self._serialiser, **params)
+        self._bindings[fun.type_name] = FlinkTask(wrapper or fun, self._serialiser, self._events, **params)
 
     def bind(
         self, 
@@ -89,7 +98,8 @@ class FlinkTasks(object):
         with_state: bool = False, 
         is_fruitful: bool = True, 
         module_name: str = None,
-        with_context: bool = False):
+        with_context: bool = False,
+        display_name: str = None):
         """
         Binds a function as a Flink Task
 
@@ -101,6 +111,7 @@ class FlinkTasks(object):
         :param module_name: if specified then the task type used in addressing will be module_name.function_name
                             otherwise the Python module containing the function will be used
         :param with_context: whether to pass a Flink context object as the first parameter (default false)
+        :param display_name: optional friendly name for this task
         """
         def wrapper(function):
             def defaults():
@@ -111,7 +122,8 @@ class FlinkTasks(object):
                     'with_state': with_state,
                     'is_fruitful': is_fruitful,
                     'module_name': module_name,
-                    'with_context': with_context
+                    'with_context': with_context,
+                    'display_name': display_name
                 }
 
             def send(*args, **kwargs):
@@ -244,7 +256,7 @@ class FlinkTasks(object):
         pipeline_protos = context.pipeline_state.pipeline
 
         if pipeline_protos is not None:
-            return _Pipeline.from_proto(pipeline_protos, self._serialiser)
+            return _Pipeline.from_proto(pipeline_protos, self._serialiser, self._events)
         else:
             raise ValueError(f'Missing pipeline for task_id - {context.get_task_id()}')
 
